@@ -14,6 +14,7 @@ import os
 import matplotlib.pyplot as plt
 import numpy as np
 import scipy.signal as spsig
+import scipy
 import copy
 import ecosound.core.tools
 
@@ -82,6 +83,12 @@ class Sound:
     tighten_waveform_window(energy_percentage)
         Crops the beginning  and end times of a waveform in a Sound object
         based on a percentage of energy.
+    upsample(resolution_sec)
+        upsample the waveform to a time resolution of resolution_sec.
+    decimate(new_sampling_frequency, filter_order=8, filter_type="iir")
+        Decimate waveform.
+    normalize()
+        Normalize max amplitude of waveform to 1.
 
     """
 
@@ -108,8 +115,9 @@ class Sound:
             myfile = sf.SoundFile(infile)
             self._file_duration_sample = myfile.seek(0, sf.SEEK_END)
             self._file_sampling_frequency = myfile.samplerate
-            self._file_duration_sec = self._file_duration_sample / \
-                self._file_sampling_frequency
+            self._file_duration_sec = (
+                self._file_duration_sample / self._file_sampling_frequency
+            )
             self._channels = myfile.channels
             self._channel_selected = []
             self._file_dir = os.path.dirname(infile)
@@ -125,10 +133,33 @@ class Sound:
             self.detrended = []
             myfile.close()
         else:
-            raise ValueError("The sound file can't be found. Please verify"
-                             + ' sound file name and path')
+            raise ValueError(
+                "The sound file can't be found. Please verify"
+                + " sound file name and path"
+            )
 
-    def read(self, channel=0, chunk=[], unit='samp', detrend=False):
+    def detrend(self):
+        self._waveform = self._waveform - np.mean(self._waveform)
+
+    def write(
+        self,
+        outfilename,
+        subtype="PCM_24",
+        endian=None,
+        format=None,
+        closefd=True,
+    ):
+        sf.write(
+            outfilename,
+            self.waveform,
+            int(self.waveform_sampling_frequency),
+            subtype=subtype,
+            endian=endian,
+            format=format,
+            closefd=closefd,
+        )
+
+    def read(self, channel=0, chunk=[], unit="samp", detrend=False):
         """
         Load data from sound file.
 
@@ -173,47 +204,78 @@ class Sound:
                 sig, fs = sf.read(self.file_full_path, always_2d=True)
                 self._waveform = sig[:, channel]
                 self._waveform_start_sample = 0
-                self._waveform_stop_sample = self.file_duration_sample-1
+                self._waveform_stop_sample = self.file_duration_sample - 1
                 self._waveform_duration_sample = len(self._waveform)
-                self._waveform_duration_sec = self._waveform_duration_sample/fs
+                self._waveform_duration_sec = (
+                    self._waveform_duration_sample / fs
+                )
             else:
-                if unit not in ('samp','sec'):
-                    raise ValueError('Invalid unit. Should be set to "sec" or'
-                                     + '"samp".')
+                if unit not in ("samp", "sec"):
+                    raise ValueError(
+                        'Invalid unit. Should be set to "sec" or' + '"samp".'
+                    )
                 # convert chunk to sampels if needed
-                if unit in ('sec'):
-                    chunk = np.round(np.dot(chunk,self.waveform_sampling_frequency))
+                if unit in ("sec"):
+                    chunk = np.round(
+                        np.dot(chunk, self.waveform_sampling_frequency)
+                    )
                 if len(chunk) == 2:  # only read a section of the file
                     # Validate input values
-                    if (chunk[0] < 0) | (chunk[0] >= self.file_duration_sample):
-                        raise ValueError('Invalid chunk start value. The sample'
-                                          + ' value chunk[0] is outside of the'
-                                          + ' file limits.')
-                    elif (chunk[1] < 0) | (chunk[1] > self.file_duration_sample):
-                        raise ValueError('Invalid chunk stop value. The sample'
-                                         + ' value chunk[1] is outside of the'
-                                         + ' file limits.')
+                    if (chunk[0] < 0) | (
+                        chunk[0] >= self.file_duration_sample
+                    ):
+                        raise ValueError(
+                            "Invalid chunk start value. The sample"
+                            + " value chunk[0] is outside of the"
+                            + " file limits."
+                        )
+                    elif (chunk[1] < 0) | (
+                        chunk[1] > self.file_duration_sample
+                    ):
+                        raise ValueError(
+                            "Invalid chunk stop value. The sample"
+                            + " value chunk[1] is outside of the"
+                            + " file limits."
+                        )
                     elif chunk[1] <= chunk[0]:
-                        raise ValueError('Invalid chunk values. chunk[1] must'
-                                         + ' be greater than chunk[0]')
+                        raise ValueError(
+                            "Invalid chunk values. chunk[1] must"
+                            + " be greater than chunk[0]"
+                        )
                     # read data
-                    sig, fs = sf.read(self.file_full_path, start=int(chunk[0]),
-                                      stop=int(chunk[1]), always_2d=True)
+                    sig, fs = sf.read(
+                        self.file_full_path,
+                        start=int(chunk[0]),
+                        stop=int(chunk[1]),
+                        always_2d=True,
+                    )
                     self._waveform = sig[:, channel]
                     self._waveform_start_sample = chunk[0]
                     self._waveform_stop_sample = chunk[1]
                     self._waveform_duration_sample = len(self._waveform)
-                    self._waveform_duration_sec = self._waveform_duration_sample/fs
+                    self._waveform_duration_sec = (
+                        self._waveform_duration_sample / fs
+                    )
                 else:
-                    raise ValueError('Invalid chunk values. The argument chunk'
-                                     + ' must be a list of 2 elements.')
+                    raise ValueError(
+                        "Invalid chunk values. The argument chunk"
+                        + " must be a list of 2 elements."
+                    )
             self._channel_selected = channel
-            if detrend: # removes DC offset
-                self._waveform = self._waveform - np.mean(self._waveform)
+            if detrend:  # removes DC offset
+                self.detrend()
+                # self._waveform = self._waveform - np.mean(self._waveform)
 
         else:
-            msg = ''.join(['Channel ', str(channel), ' does not exist (',
-                           str(self._channels), ' channels available).'])
+            msg = "".join(
+                [
+                    "Channel ",
+                    str(channel),
+                    " does not exist (",
+                    str(self._channels),
+                    " channels available).",
+                ]
+            )
             raise ValueError(msg)
 
     def filter(self, filter_type, cutoff_frequencies, order=4, verbose=True):
@@ -256,25 +318,35 @@ class Sound:
         if self._filter_applied is False:
 
             # check bandpass cuttoff freq and switch to lowpass.highpass if necessary
-            if (filter_type == 'bandpass') and (min(cutoff_frequencies) <=0):
+            if (filter_type == "bandpass") and (min(cutoff_frequencies) <= 0):
                 cutoff_frequencies = [max(cutoff_frequencies)]
-                filter_type = 'lowpass'
+                filter_type = "lowpass"
                 if verbose:
-                    print('Warning: filter type was changed from "bandpass" to "lowpass".')
-            if (filter_type == 'bandpass') and (max(cutoff_frequencies) >=self._waveform_sampling_frequency/2):
+                    print(
+                        'Warning: filter type was changed from "bandpass" to "lowpass".'
+                    )
+            if (filter_type == "bandpass") and (
+                max(cutoff_frequencies)
+                >= self._waveform_sampling_frequency / 2
+            ):
                 cutoff_frequencies = [min(cutoff_frequencies)]
-                filter_type = 'highpass'
+                filter_type = "highpass"
                 if verbose:
-                    print('Warning: filter type was changed from "bandpass" to "highpass".')
+                    print(
+                        'Warning: filter type was changed from "bandpass" to "highpass".'
+                    )
             # Instantiate filter object
             my_filter = Filter(filter_type, cutoff_frequencies, order)
-            self._waveform = my_filter.apply(self._waveform,
-                                             self._waveform_sampling_frequency)
+            self._waveform = my_filter.apply(
+                self._waveform, self._waveform_sampling_frequency
+            )
             self._filter_applied = True
             self._filter_params = my_filter
         else:
-            raise ValueError('This signal has been filtered already. Cannot'
-                             + ' filter twice.')
+            raise ValueError(
+                "This signal has been filtered already. Cannot"
+                + " filter twice."
+            )
 
     def upsample(self, resolution_sec):
         """
@@ -290,19 +362,84 @@ class Sound:
 
         Returns
         -------
-        None. Updates the waveform of the Sound object.
+        None. Updates the waveform and sampling frequency of the Sound object.
 
         """
         self._waveform, self._waveform_sampling_frequency = upsample(
             self._waveform,
-            1/ self._waveform_sampling_frequency,
-            resolution_sec)
+            1 / self._waveform_sampling_frequency,
+            resolution_sec,
+        )
+        self._waveform_duration_sec = (
+            len(self._waveform) / self._waveform_sampling_frequency
+        )
+        self._waveform_duration_sample = (
+            self._waveform_duration_sec * self._waveform_sampling_frequency
+        )
 
-    def normalize(self, method='amplitude'):
-        if method == 'amplitude':
+    def decimate(
+        self, new_sampling_frequency, filter_order=8, filter_type="iir"
+    ):
+        """
+        Decimate  waveform
+
+        Filter and reduce the number of samples in the waveform.
+
+        Parameters
+        ----------
+        new_sampling_frequency : float
+            Sampling frequency requested, in Hz.
+        filter_order : int, optional
+            Order of the low-pass filter to use. The default is 8.
+        filter_type : str, optional
+            Type of low-pass filter to use. The default is 'iir'.
+
+        Returns
+        -------
+        None. Updates the waveform and sampling frequency of the Sound object.
+
+        """
+
+        # downsample to user-defined sampling rate
+        downsampling_factor = int(
+            np.round(self.waveform_sampling_frequency / new_sampling_frequency)
+        )
+
+        # decimate signal (the cutoff frequency of the filter is 0.8 x new_sampling_frequency)
+        sig_decimated = scipy.signal.decimate(
+            self.waveform,
+            downsampling_factor,
+            n=filter_order,
+            ftype=filter_type,
+            axis=0,
+            zero_phase=True,
+        )
+        # update object
+        self._waveform = sig_decimated
+        self._waveform_sampling_frequency = (
+            self.waveform_sampling_frequency / downsampling_factor
+        )
+        self._waveform_duration_sec = (
+            len(sig_decimated) / self._waveform_sampling_frequency
+        )
+        self._waveform_duration_sample = (
+            self._waveform_duration_sec * self._waveform_sampling_frequency
+        )
+
+    def normalize(self, method="amplitude"):
+        if method == "amplitude":
             self._waveform = self._waveform / np.max(self._waveform)
 
-    def plot(self, unit='sec', newfig=False, label=[],linestyle='-', marker='',color='black', title=''):
+    def plot(
+        self,
+        unit="sec",
+        newfig=False,
+        label=[],
+        linestyle="-",
+        marker="",
+        color="black",
+        title="",
+    ):
         """
         Plot waveform of the audio signal.
 
@@ -335,35 +472,41 @@ class Sound:
 
         """
         if len(self._waveform) == 0:
-            raise ValueError('Cannot plot, waveform data enpty. Use Sound.read'
-                             + ' to load the waveform')
-        if unit == 'sec':
-            axis_t = np.arange(0, len(self._waveform)
-                               / self._waveform_sampling_frequency, 1
-                               / self._waveform_sampling_frequency)
-            xlabel = 'Time (sec)'
-        elif unit == 'samp':
+            raise ValueError(
+                "Cannot plot, waveform data enpty. Use Sound.read"
+                + " to load the waveform"
+            )
+        if unit == "sec":
+            axis_t = np.arange(
+                0,
+                len(self._waveform) / self._waveform_sampling_frequency,
+                1 / self._waveform_sampling_frequency,
+            )
+            xlabel = "Time (sec)"
+        elif unit == "samp":
             axis_t = np.arange(0, len(self._waveform), 1)
-            xlabel = 'Time (sample)'
+            xlabel = "Time (sample)"
         if newfig:
             plt.figure()
-        axis_t = axis_t[0:len(self._waveform)]
-        plt.plot(axis_t, self._waveform,
-                           color=color,
-                           marker = marker,
-                           linestyle = linestyle,
-                           label=label,
-                           )
+        axis_t = axis_t[0 : len(self._waveform)]
+        plt.plot(
+            axis_t,
+            self._waveform,
+            color=color,
+            marker=marker,
+            linestyle=linestyle,
+            label=label,
+        )
         plt.xlabel(xlabel)
-        plt.ylabel('Amplitude')
+        plt.ylabel("Amplitude")
         plt.title(title)
-        plt.axis([axis_t[0], axis_t[-1],
-                  min(self._waveform),
-                  max(self._waveform)])
+        plt.axis(
+            [axis_t[0], axis_t[-1], min(self._waveform), max(self._waveform)]
+        )
         plt.grid()
         plt.show()
 
-    def select_snippet(self, chunk, unit='samp'):
+    def select_snippet(self, chunk, unit="samp"):
         """
         Select section of the loaded waveform.
 
@@ -393,31 +536,46 @@ class Sound:
 
         """
         if len(chunk) != 2:
-            raise ValueError('Chunk should be a list of with 2 values: '
-                             + 'chunk=[t1, t2].')
-        elif unit not in ('samp','sec'):
-           raise ValueError('Invalid unit. Should be set to "sec" or "samp".')
+            raise ValueError(
+                "Chunk should be a list of with 2 values: " + "chunk=[t1, t2]."
+            )
+        elif unit not in ("samp", "sec"):
+            raise ValueError('Invalid unit. Should be set to "sec" or "samp".')
         elif chunk[0] >= chunk[1]:
-            raise ValueError('Chunk[0] should be greater than chunk[1].')
+            raise ValueError("Chunk[0] should be greater than chunk[1].")
 
-        if unit == 'sec':
-            chunk[0] = int(np.floor(chunk[0] * self.waveform_sampling_frequency))
-            chunk[1] = int(np.ceil(chunk[1] * self.waveform_sampling_frequency))
+        if unit == "sec":
+            chunk[0] = int(
+                np.floor(chunk[0] * self.waveform_sampling_frequency)
+            )
+            chunk[1] = int(
+                np.ceil(chunk[1] * self.waveform_sampling_frequency)
+            )
 
         if (chunk[0] < 0) | (chunk[0] > self.file_duration_sample):
-            raise ValueError('Invalid chunk start value. The start value '
-                             + 'chunk[0] is outside of file limit.')
+            raise ValueError(
+                "Invalid chunk start value. The start value "
+                + "chunk[0] is outside of file limit."
+            )
         elif (chunk[1] < 0) | (chunk[1] > self.file_duration_sample):
-            raise ValueError('Invalid chunk stop value. The stop value '
-                             + 'chunk[1] is outside of file limit.')
-
+            raise ValueError(
+                "Invalid chunk stop value. The stop value "
+                + "chunk[1] is outside of file limit."
+            )
 
         snippet = copy.deepcopy(self)
-        snippet._waveform = self._waveform[chunk[0]:chunk[1]]
-        snippet._waveform_stop_sample = snippet._waveform_start_sample + chunk[1]
-        snippet._waveform_start_sample = snippet._waveform_start_sample + chunk[0]
+        snippet._waveform = self._waveform[chunk[0] : chunk[1]]
+        snippet._waveform_stop_sample = (
+            snippet._waveform_start_sample + chunk[1]
+        )
+        snippet._waveform_start_sample = (
+            snippet._waveform_start_sample + chunk[0]
+        )
         snippet._waveform_duration_sample = len(snippet._waveform)
-        snippet._waveform_duration_sec = snippet._waveform_duration_sec / snippet._waveform_sampling_frequency
+        snippet._waveform_duration_sec = (
+            snippet._waveform_duration_sec
+            / snippet._waveform_sampling_frequency
+        )
         return snippet
 
     def tighten_waveform_window(self, energy_percentage):
@@ -440,7 +598,9 @@ class Sound:
         -related attributes.
 
         """
-        chunk = ecosound.core.tools.tighten_signal_limits(self._waveform, energy_percentage)
+        chunk = ecosound.core.tools.tighten_signal_limits(
+            self._waveform, energy_percentage
+        )
         snip = self.select_snippet(chunk)
         self.__dict__.update(snip.__dict__)
 
@@ -486,7 +646,10 @@ class Sound:
     @property
     def file_full_path(self):
         """Return the file_full_path attribute."""
-        return os.path.join(self._file_dir, self._file_name) + self._file_extension
+        return (
+            os.path.join(self._file_dir, self._file_name)
+            + self._file_extension
+        )
 
     @property
     def file_extension(self):
@@ -592,24 +755,34 @@ class Filter:
 
         """
         # chech filter type
-        if (type == 'bandpass') | (type == 'lowpass') | (type == 'highpass') == 0:
-            raise ValueError('Wrong filter type. Must be "bandpass", "lowpass"'
-                             +', or "highpass".')
+        if (type == "bandpass") | (type == "lowpass") | (
+            type == "highpass"
+        ) == 0:
+            raise ValueError(
+                'Wrong filter type. Must be "bandpass", "lowpass"'
+                + ', or "highpass".'
+            )
         # chech freq values
-        if (type == 'bandpass'):
+        if type == "bandpass":
             if len(cutoff_frequencies) != 2:
-                raise ValueError('The type "bandpass" requires two frepuency '
-                                 + 'values: cutoff_frequencies=[lowcut, '
-                                 + 'highcut].')
+                raise ValueError(
+                    'The type "bandpass" requires two frepuency '
+                    + "values: cutoff_frequencies=[lowcut, "
+                    + "highcut]."
+                )
             elif cutoff_frequencies[0] > cutoff_frequencies[1]:
-                raise ValueError('The lowcut value should be smaller than the '
-                                 + 'highcut value: cutoff_frequencies=[lowcut,'
-                                 + ' highcut].')
-        elif (type == 'lowpass') | (type == 'highpass'):
+                raise ValueError(
+                    "The lowcut value should be smaller than the "
+                    + "highcut value: cutoff_frequencies=[lowcut,"
+                    + " highcut]."
+                )
+        elif (type == "lowpass") | (type == "highpass"):
             if len(cutoff_frequencies) != 1:
-                raise ValueError('The type "lowpass" and "highpass" require '
-                                 + 'one frequency value cutoff_frequencies='
-                                 + '[cutfreq].')
+                raise ValueError(
+                    'The type "lowpass" and "highpass" require '
+                    + "one frequency value cutoff_frequencies="
+                    + "[cutfreq]."
+                )
         self.type = type
         self.cutoff_frequencies = cutoff_frequencies
         self.order = order
@@ -631,10 +804,10 @@ class Filter:
             Filtered time series.
 
         """
-        #b, a = self.coefficients(sampling_frequency)
-        #return spsig.sosfiltfilt (b, a, waveform)
+        # b, a = self.coefficients(sampling_frequency)
+        # return spsig.sosfiltfilt (b, a, waveform)
         sos = self.coefficients(sampling_frequency)
-        return spsig.sosfiltfilt (sos, waveform)
+        return spsig.sosfiltfilt(sos, waveform)
 
     def coefficients(self, sampling_frequency):
         """
@@ -654,52 +827,63 @@ class Filter:
 
         """
         nyquist = 0.5 * sampling_frequency
-        if self.type == 'bandpass':
+        if self.type == "bandpass":
             low = self.cutoff_frequencies[0] / nyquist
             high = self.cutoff_frequencies[1] / nyquist
-            #b, a = spsig.butter(self.order, [low, high], btype='band')
-            sos = spsig.butter(self.order, [low, high], btype='band', output='sos')
-        elif self.type == 'lowpass':
+            # b, a = spsig.butter(self.order, [low, high], btype='band')
+            sos = spsig.butter(
+                self.order, [low, high], btype="band", output="sos"
+            )
+        elif self.type == "lowpass":
             # b, a = spsig.butter(self.order,
             #                     self.cutoff_frequencies[0]/nyquist, 'low')
-            sos = spsig.butter(self.order,
-                                self.cutoff_frequencies[0]/nyquist, 'low',output='sos')
-        elif self.type == 'highpass':
+            sos = spsig.butter(
+                self.order,
+                self.cutoff_frequencies[0] / nyquist,
+                "low",
+                output="sos",
+            )
+        elif self.type == "highpass":
             # b, a = spsig.butter(self.order,
             #                     self.cutoff_frequencies[0]/nyquist, 'high')
-            sos = spsig.butter(self.order,
-                                self.cutoff_frequencies[0]/nyquist, 'high',output='sos')
+            sos = spsig.butter(
+                self.order,
+                self.cutoff_frequencies[0] / nyquist,
+                "high",
+                output="sos",
+            )
         return sos
 
 
 def upsample(waveform, current_res_sec, new_res_sec):
-        """
-        Upsample  waveform
+    """
+    Upsample  waveform
 
-        Increase the number of samples in the waveform and interpolate.
+    Increase the number of samples in the waveform and interpolate.
 
-        Parameters
-        ----------
-        waveform: 1D array
-            Waveform to upsample
-        current_res_sec : float
-            Time resolution of waveform in seconds. It is the inverse of the
-            sampling frequency.
-        new_res_sec : float
-            New time resolution of waveform after interpolation (in seconds).
+    Parameters
+    ----------
+    waveform: 1D array
+        Waveform to upsample
+    current_res_sec : float
+        Time resolution of waveform in seconds. It is the inverse of the
+        sampling frequency.
+    new_res_sec : float
+        New time resolution of waveform after interpolation (in seconds).
 
-        Returns
-        -------
-        waveform: 1D array
-            waveform upsampled to have a time resolution of "new_res_sec".
+    Returns
+    -------
+    waveform: 1D array
+        waveform upsampled to have a time resolution of "new_res_sec".
 
-        """
-        axis_t = np.arange(0, len(waveform)*current_res_sec, current_res_sec)
-        new_fs = round(1/new_res_sec)
-        nb_samp = round(axis_t[-1]*new_fs)
-        new_waveform, new_axis_t = spsig.resample(waveform,
-                                                  nb_samp,
-                                                  t=axis_t,
-                                                  window='hann',
-                                                  )
-        return new_waveform, new_fs
\ No newline at end of file
+    """
+    axis_t = np.arange(0, len(waveform) * current_res_sec, current_res_sec)
+    new_fs = round(1 / new_res_sec)
+    nb_samp = round(axis_t[-1] * new_fs)
+    new_waveform, new_axis_t = spsig.resample(
+        waveform,
+        nb_samp,
+        t=axis_t,
+        window="hann",
+    )
+    return new_waveform, new_fs
