@@ -19,6 +19,7 @@ import dask_image.ndfilters
 
 ## TODO: change Asserts by Raise
 
+
 class Spectrogram:
     """A class for spectrograms.
 
@@ -71,9 +72,19 @@ class Spectrogram:
         'median_equalizer':    window_duration in seconds.
     """
 
-    _valid_units = ('samp', 'sec')
-    _valid_windows = ('hann',)
-    def __init__(self, frame, window_type, fft, step, sampling_frequency, unit='sec', verbose=True):
+    _valid_units = ("samp", "sec")
+    _valid_windows = ("hann",)
+
+    def __init__(
+        self,
+        frame,
+        window_type,
+        fft,
+        step,
+        sampling_frequency,
+        unit="sec",
+        verbose=True,
+    ):
         """
         Initialize Spectrogram object.
 
@@ -109,17 +120,32 @@ class Spectrogram:
 
         """
         # Validation of the imput parameters
-        assert (unit in Spectrogram._valid_units), ("Wrong unit value. Valid \
-                                           units: ", Spectrogram._valid_units)
+        assert unit in Spectrogram._valid_units, (
+            "Wrong unit value. Valid \
+                                           units: ",
+            Spectrogram._valid_units,
+        )
         assert fft >= frame, " fft should alwyas be >= frame"
         assert step < frame, "step should always be <= frame"
-        assert (window_type in Spectrogram._valid_windows), ("Wrong window type\
-                                . Valid values: ", Spectrogram._valid_windows)
+        assert window_type in Spectrogram._valid_windows, (
+            "Wrong window type\
+                                . Valid values: ",
+            Spectrogram._valid_windows,
+        )
 
         # Convert units in seconds/samples
-        self._frame_samp, self._fft_samp, self._step_samp, self._frame_sec,\
-        self._fft_sec, self._step_sec, self._overlap_perc, self._overlap_samp =\
-        Spectrogram._convert_units(frame, fft, step, sampling_frequency, unit, verbose=verbose)
+        (
+            self._frame_samp,
+            self._fft_samp,
+            self._step_samp,
+            self._frame_sec,
+            self._fft_sec,
+            self._step_sec,
+            self._overlap_perc,
+            self._overlap_samp,
+        ) = Spectrogram._convert_units(
+            frame, fft, step, sampling_frequency, unit, verbose=verbose
+        )
 
         # Time and frequency resolution
         self._sampling_frequency = sampling_frequency
@@ -132,26 +158,39 @@ class Spectrogram:
         self._axis_frequencies = []
         self._axis_times = []
 
-    def _convert_units(frame, fft, step, sampling_frequency, unit, verbose=True):
+    def _convert_units(
+        frame, fft, step, sampling_frequency, unit, verbose=True
+    ):
         """Convert frame, fft, and step values to samples/seconds"""
-        if unit == 'sec':
-            frame_samp = round(frame*sampling_frequency)
-            fft_samp = adjust_FFT_size(round(fft*sampling_frequency),verbose=verbose)
-            step_samp = round(step*sampling_frequency)
+        if unit == "sec":
+            frame_samp = round(frame * sampling_frequency)
+            fft_samp = adjust_FFT_size(
+                round(fft * sampling_frequency), verbose=verbose
+            )
+            step_samp = round(step * sampling_frequency)
             frame_sec = frame
-            fft_sec = fft_samp*sampling_frequency
+            fft_sec = fft_samp * sampling_frequency
             step_sec = step
-        elif unit == 'samp':
+        elif unit == "samp":
             frame_samp = frame
-            fft_samp = adjust_FFT_size(fft,verbose=verbose)
+            fft_samp = adjust_FFT_size(fft, verbose=verbose)
             step_samp = step
-            frame_sec = frame/sampling_frequency
-            fft_sec = fft_samp/sampling_frequency
-            step_sec = step/sampling_frequency
+            frame_sec = frame / sampling_frequency
+            fft_sec = fft_samp / sampling_frequency
+            step_sec = step / sampling_frequency
 
-        overlap_samp = frame_samp-step_samp
-        overlap_perc = (overlap_samp/frame_samp)*100
-        return frame_samp, fft_samp, step_samp, frame_sec, fft_sec, step_sec, overlap_perc,overlap_samp
+        overlap_samp = frame_samp - step_samp
+        overlap_perc = (overlap_samp / frame_samp) * 100
+        return (
+            frame_samp,
+            fft_samp,
+            step_samp,
+            frame_sec,
+            fft_sec,
+            step_sec,
+            overlap_perc,
+            overlap_samp,
+        )
 
     def _compute_old(self, sig):
         """
@@ -176,13 +215,26 @@ class Spectrogram:
             2-D array with spectrogram values.
 
         """
-        assert sig.waveform_sampling_frequency == self.sampling_frequency, "The sampling frequency provided doesn't match the one from the Spectrogram object."
+        assert (
+            sig.waveform_sampling_frequency == self.sampling_frequency
+        ), "The sampling frequency provided doesn't match the one from the Spectrogram object."
         # Weighting window
-        if self.window_type == 'hann':
+        if self.window_type == "hann":
             window = signal.hann(self.frame_samp)
         # Calculates  spectrogram
-        self._axis_frequencies, self._axis_times, self._spectrogram = signal.spectrogram(sig.waveform, fs=self.sampling_frequency, window=window, noverlap=self.overlap_samp, nfft=self.fft_samp, scaling='spectrum')
-        self._spectrogram = 20*np.log10(self._spectrogram)
+        (
+            self._axis_frequencies,
+            self._axis_times,
+            self._spectrogram,
+        ) = signal.spectrogram(
+            sig.waveform,
+            fs=self.sampling_frequency,
+            window=window,
+            noverlap=self.overlap_samp,
+            nfft=self.fft_samp,
+            scaling="spectrum",
+        )
+        self._spectrogram = 20 * np.log10(self._spectrogram)
         return self._axis_frequencies, self._axis_times, self._spectrogram
 
     def compute(self, sig, dB=False, use_dask=False, dask_chunks=40):
@@ -218,63 +270,98 @@ class Spectrogram:
             2-D array with spectrogram values.
         """
         # Weighting window
-        if self.window_type == 'hann':
+        if self.window_type == "hann":
             win = np.hanning(self.frame_samp)
-        #sig = self.waveform
-        #step = self.frame_samp - self.overlap_samp
-        starts = np.arange(0,len(sig.waveform)-self.frame_samp, self.step_samp, dtype=int)
-        stops = starts + self.frame_samp
+        # if signal is shorter than spectrogram frame -> add zeros
+        if len(sig.waveform) < self.frame_samp:
+            vec = np.zeros(self.frame_samp)
+            vec[0 : len(sig.waveform)] = sig.waveform
+            sig._waveform = vec
+            sig._waveform_duration_sample = len(vec)
+            sig._waveform_duration_sec = (
+                sig._waveform_duration_sample / sig.waveform_sampling_frequency
+            )
+            starts = np.array([0])
+            stops = np.array([len(vec)])
+        else:
+            starts = np.arange(
+                0,
+                len(sig.waveform) - self.frame_samp,
+                self.step_samp,
+                dtype=int,
+            )
+            stops = starts + self.frame_samp
         start_chunks = np.array_split(starts, dask_chunks)
         stop_chunks = np.array_split(stops, dask_chunks)
+
         spectrogram = []
-        idx=0
+        idx = 0
         for start_chunk, stop_chunk in zip(start_chunks, stop_chunks):
-            sig_chunk = sig.waveform[start_chunk[0]:stop_chunk[-1]]
-            chunk_size = len(start_chunk)
-            if use_dask:
-                spectro_chunk = delayed(Spectrogram._calc_spectrogram)(sig_chunk,win,
-                                           start_chunk-start_chunk[0],
-                                           stop_chunk-start_chunk[0],
-                                           self.fft_samp)
-            else:
-                spectro_chunk = Spectrogram._calc_spectrogram(sig_chunk,win,
-                                start_chunk-start_chunk[0],
-                                stop_chunk-start_chunk[0],
-                                self.fft_samp)
-            spectrogram.append(spectro_chunk)
-            idx += chunk_size
+            if (len(start_chunk) > 0) & (len(stop_chunk) > 0):
+                # print(start_chunk)
+                # print(stop_chunk)
+                # print("---------")
+                sig_chunk = sig.waveform[start_chunk[0] : stop_chunk[-1]]
+                chunk_size = len(start_chunk)
+                if use_dask:
+                    spectro_chunk = delayed(Spectrogram._calc_spectrogram)(
+                        sig_chunk,
+                        win,
+                        start_chunk - start_chunk[0],
+                        stop_chunk - start_chunk[0],
+                        self.fft_samp,
+                    )
+                else:
+                    spectro_chunk = Spectrogram._calc_spectrogram(
+                        sig_chunk,
+                        win,
+                        start_chunk - start_chunk[0],
+                        stop_chunk - start_chunk[0],
+                        self.fft_samp,
+                    )
+                spectrogram.append(spectro_chunk)
+                idx += chunk_size
         if use_dask:
             spectrogram = compute(spectrogram)
             spectrogram = np.concatenate(spectrogram[0][:], axis=1)
         else:
             spectrogram = np.concatenate(spectrogram[:], axis=1)
         if dB:
-            spectrogram = 20*np.log10(spectrogram)
+            spectrogram = 20 * np.log10(spectrogram)
         self._spectrogram = spectrogram
-        self._axis_times = starts/self.sampling_frequency  # ?? needed ?
-        self._axis_frequencies = np.arange(0,self._sampling_frequency/2,self._frequency_resolution) # ?? needed ?
+        self._axis_times = starts / self.sampling_frequency  # ?? needed ?
+        self._axis_frequencies = np.arange(
+            0, self._sampling_frequency / 2, self._frequency_resolution
+        )  # ?? needed ?
         return self._axis_frequencies, self._axis_times, self._spectrogram
 
     @staticmethod
-    #@njit
+    # @njit
     def _calc_spectrogram(sig, win, starts, stops, fft_samp):
         fft_samp = np.int32(fft_samp)
-        fnyq0 = np.round(fft_samp/2)
+        fnyq0 = np.round(fft_samp / 2)
         fnyq = np.int(fnyq0)
         spectro = np.empty((fnyq, len(starts)))  # the default
         idx = 0
         for start, stop in zip(starts, stops):
-            s = sig[start:stop]*win
+            s = sig[start:stop] * win
             Spectrum = np.fft.fft(s, fft_samp)
             Spectrum = abs(Spectrum)  # amplitude
-            Spectrum = Spectrum*2
+            Spectrum = Spectrum * 2
             # Spectrum = Spectrum**2
             # ts_window = getFFT(sig[start:stop],fft_samp)
             spectro[:, idx] = Spectrum[0:fnyq]
             idx += 1
         return spectro
 
-    def crop(self, frequency_min=None, frequency_max=None, time_min=None, time_max=None, inplace=False):
+    def crop(
+        self,
+        frequency_min=None,
+        frequency_max=None,
+        time_min=None,
+        time_max=None,
+        inplace=False,
+    ):
         """
         Crop frequencies from the spectrogram.
 
@@ -313,11 +400,11 @@ class Spectrogram:
             else:
                 min_row_idx = min_row_idx[0][-1] + 1
         if frequency_max is None:
-            max_row_idx = self._axis_frequencies.size-1
+            max_row_idx = self._axis_frequencies.size - 1
         else:
             max_row_idx = np.where(self._axis_frequencies > frequency_max)
             if np.size(max_row_idx) == 0:
-                max_row_idx = self._axis_frequencies.size-1
+                max_row_idx = self._axis_frequencies.size - 1
             else:
                 max_row_idx = max_row_idx[0][0]
         # Find time indices
@@ -330,11 +417,11 @@ class Spectrogram:
             else:
                 min_col_idx = min_col_idx[0][-1] + 1
         if time_max is None:
-            max_col_idx = self._axis_times.size-1
+            max_col_idx = self._axis_times.size - 1
         else:
             max_col_idx = np.where(self._axis_times > time_max)
             if np.size(max_col_idx) == 0:
-                max_col_idx = self._axis_times.size-1
+                max_col_idx = self._axis_times.size - 1
             else:
                 max_col_idx = max_col_idx[0][0]
         # update spectrogram and axes
@@ -343,19 +430,35 @@ class Spectrogram:
             # self._axis_times = np.arange(0,(max_col_idx - min_col_idx)*self._time_resolution,self._time_resolution)
             # self._spectrogram = self._spectrogram[min_row_idx:max_row_idx, min_col_idx:max_col_idx]
             # out_object = None
-            self._axis_frequencies = self._axis_frequencies[min_row_idx:max_row_idx+1]
-            self._axis_times = self._axis_times[min_col_idx:max_col_idx+1]-self._axis_times[min_col_idx]
-            self._spectrogram = self._spectrogram[min_row_idx:max_row_idx+1, min_col_idx:max_col_idx+1]
+            self._axis_frequencies = self._axis_frequencies[
+                min_row_idx : max_row_idx + 1
+            ]
+            self._axis_times = (
+                self._axis_times[min_col_idx : max_col_idx + 1]
+                - self._axis_times[min_col_idx]
+            )
+            self._spectrogram = self._spectrogram[
+                min_row_idx : max_row_idx + 1, min_col_idx : max_col_idx + 1
+            ]
             out_object = None
         else:
             out_object = copy.copy(self)
-            out_object._axis_frequencies = out_object._axis_frequencies[min_row_idx:max_row_idx+1]
-            #out_object._axis_times = np.arange(0,(max_col_idx - min_col_idx)*out_object._time_resolution,out_object._time_resolution)
-            out_object._axis_times = out_object._axis_times[min_col_idx:max_col_idx+1]-out_object._axis_times[min_col_idx]
-            #out_object._spectrogram = out_object._spectrogram[min_row_idx:max_row_idx, min_col_idx:max_col_idx]
-            out_object._spectrogram = out_object._spectrogram[min_row_idx:max_row_idx+1, min_col_idx:max_col_idx+1]
+            out_object._axis_frequencies = out_object._axis_frequencies[
+                min_row_idx : max_row_idx + 1
+            ]
+            # out_object._axis_times = np.arange(0,(max_col_idx - min_col_idx)*out_object._time_resolution,out_object._time_resolution)
+            out_object._axis_times = (
+                out_object._axis_times[min_col_idx : max_col_idx + 1]
+                - out_object._axis_times[min_col_idx]
+            )
+            # out_object._spectrogram = out_object._spectrogram[min_row_idx:max_row_idx, min_col_idx:max_col_idx]
+            out_object._spectrogram = out_object._spectrogram[
+                min_row_idx : max_row_idx + 1, min_col_idx : max_col_idx + 1
+            ]
             if out_object._spectrogram.shape[1] != len(out_object._axis_times):
-                raise ValueError("Spectrogram axes don't match spectrogram matrix.")
+                raise ValueError(
+                    "Spectrogram axes don't match spectrogram matrix."
+                )
         return out_object
 
     def denoise(self, method, **kwargs):
@@ -385,14 +488,22 @@ class Spectrogram:
         None. Denoised spectrogram matrix.
 
         """
-        denoise_methods = ('median_equalizer',)
+        denoise_methods = ("median_equalizer",)
         if method in denoise_methods:
             eval("self._" + method + "(**kwargs)")
         else:
-            raise ValueError('Method not recognized. Methods available:'
-                             + str(denoise_methods))
+            raise ValueError(
+                "Method not recognized. Methods available:"
+                + str(denoise_methods)
+            )
 
-    def _median_equalizer(self, window_duration, use_dask=False, dask_chunks=(1000,1000), inplace=False):
+    def _median_equalizer(
+        self,
+        window_duration,
+        use_dask=False,
+        dask_chunks=(1000, 1000),
+        inplace=False,
+    ):
         """
         Median equalizer.
 
@@ -422,20 +533,27 @@ class Spectrogram:
 
         """
         if use_dask:
-            dask_spectro = dask.array.from_array(self._spectrogram, chunks=dask_chunks)
-            Smed = dask_image.ndfilters.median_filter(dask_spectro,
-                                                       size=(1,round(window_duration/self.time_resolution)),
-                                                       mode='mirror')
+            dask_spectro = dask.array.from_array(
+                self._spectrogram, chunks=dask_chunks
+            )
+            Smed = dask_image.ndfilters.median_filter(
+                dask_spectro,
+                size=(1, round(window_duration / self.time_resolution)),
+                mode="mirror",
+            )
             Smed = Smed.compute()
         else:
-            Smed = ndimage.median_filter(self._spectrogram, (1,round(window_duration/self.time_resolution)))
+            Smed = ndimage.median_filter(
+                self._spectrogram,
+                (1, round(window_duration / self.time_resolution)),
+            )
         if inplace:
-            self._spectrogram = self._spectrogram-Smed
+            self._spectrogram = self._spectrogram - Smed
             self._spectrogram[self._spectrogram < 0] = 0  # floor
             out_object = None
         else:
             out_object = copy.copy(self)
-            out_object._spectrogram = out_object._spectrogram-Smed
+            out_object._spectrogram = out_object._spectrogram - Smed
             out_object._spectrogram[out_object._spectrogram < 0] = 0  # floor
         return out_object
 
@@ -514,14 +632,22 @@ class Spectrogram:
         """Return the spectrogram attribute."""
         return self._spectrogram
 
+
 def adjust_FFT_size(nfft, verbose=True):
-        """ Adjust nfft to the next power of two if necessary."""
-        nfft_adjusted = next_power_of_2(nfft)
-        if nfft_adjusted != nfft:
-            if verbose:
-                print('Warning: FFT size automatically adjusted to', nfft_adjusted, 'samples (original size:', nfft,')')
-        return nfft_adjusted
+    """Adjust nfft to the next power of two if necessary."""
+    nfft_adjusted = next_power_of_2(nfft)
+    if nfft_adjusted != nfft:
+        if verbose:
+            print(
+                "Warning: FFT size automatically adjusted to",
+                nfft_adjusted,
+                "samples (original size:",
+                nfft,
+                ")",
+            )
+    return nfft_adjusted
+
 
 def next_power_of_2(x):
     """Calculate the next power of two for x."""
-    return 1 if x == 0 else 2**(x - 1).bit_length()
+    return 1 if x == 0 else 2 ** (x - 1).bit_length()
